﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Origin;

namespace SampleAppC
{
    public partial class MainWindow : Form
    {
        static object Default = System.Type.Missing;
		static int[] sampleRates = { 1000, 2000, 5000 }; // Hertz (Hz)
		static double[] noiseLevels = { 0, 0.01, 0.02, 0.05 };
		static double numSeconds = 2;

		// The following members are initialized on click "Write OPJ" button.
		static int frequency;
		static double xIncrement;

        private static Origin.Application app;

        public MainWindow()
        {
            InitializeComponent();

			// Initialize the user settings
			comboBox_sampleRate.SelectedIndex = 0;
			comboBox_channels.SelectedIndex = 0;
            comboBox_noise.SelectedIndex = 0;
        }

        private void PrepareOriginWorkbook()
        {
			// start with a new project
			app.NewProject();
			
			// create a workbook and worksheet
			WorksheetPage wbk = app.WorksheetPages.Add(Default, Default);
			Worksheet wks = (Worksheet)wbk.Layers[0];

			// if user wants an X column for time...
			if (checkBox_xColumn.Checked)
			{
				// create the X (time) column
				Column colx = wks.Columns.Add(Default);
				colx.Type = COLTYPES.COLTYPE_X;
				colx.DataFormat = COLDATAFORMAT.DF_DOUBLE;
				colx.LongName = "Time";
			}

			// create a Y column for each channel
			for (int i = 0; i <= comboBox_channels.SelectedIndex; i++)
			{
				Column coly = wks.Columns.Add(Default);
				coly.Type = COLTYPES.COLTYPE_Y;
				coly.DataFormat = COLDATAFORMAT.DF_DOUBLE;
				coly.LongName = "Ch" + (i + 1);

				if (!checkBox_xColumn.Checked)
					coly.SetEvenSampling(0.0, xIncrement, Default, "Time", Default);
			}
        }

		private double SendDataToOriginWorkbook(double dStartTime)
		{
			int nNumChannels = comboBox_channels.SelectedIndex + 1;

			int nNumCols = nNumChannels, nFirstYCol = 0;
			if (checkBox_xColumn.Checked)
			{
				nNumCols++;
				nFirstYCol = 1;
			}

			int nNumRows = (int)(frequency * numSeconds);

			Random rand = new Random();
			double noise, noiseLevel = noiseLevels[comboBox_noise.SelectedIndex];

			double dTime = dStartTime;
			double[,] data = new double[nNumRows, nNumCols];
			for (int nRow = 0; nRow < nNumRows; nRow++)
			{
				if (nNumChannels != nNumCols)
					data[nRow, 0] = dTime / frequency;

				noise = rand.NextDouble() * noiseLevel;

				// Channel 1, Sine wave
				data[nRow, nFirstYCol] = Waveform(dTime, 0) + noise;

				// Channel 2, Square wave
				if (nNumChannels > 1)
					data[nRow, nFirstYCol + 1] = Waveform(dTime, 1) + noise;

				// Channel 3, Sawtooth wave
				if (nNumChannels > 2)
					data[nRow, nFirstYCol + 2] = Waveform(dTime, 2) + noise;

				dTime += xIncrement;
			}

			// Put the data into the active worksheet.
			app.PutWorksheet("", data, -1, 0);

			// Return next starting time.
			return dTime;
		}

		private double Waveform(double time, int waveType)
        {
            double data = 0.0;
            switch (waveType)
            {
            case 0: // Sine
				data = Math.Sin(2 * Math.PI * time);
                break;
            case 1: // Square
				data = Math.Sin(2 * Math.PI * time);
				if (data != 0.0)
					data = (data > 0.0) ? 1.0 : -1.0;
				break;
            case 2: // Sawtooth
				data = 2 * ((time - Math.Floor(time + 0.5)));
                break;
            }
            return data;
        }

		private void button_writeOPJ_Click(object sender, EventArgs e)
		{
			frequency = sampleRates[comboBox_sampleRate.SelectedIndex];
			xIncrement = 1.0 / (double)frequency;

			// Connect to Origin.
			app = new Origin.Application();

			PrepareOriginWorkbook();

			double dTime = 0.0;
			for (int nCycle = 0; nCycle < 8; nCycle++)
				dTime = SendDataToOriginWorkbook(dTime);

			// Write Origin project to specified file.
			if (app.Save(textBox_fileName.Text) == false)
				MessageBox.Show("Save failed.");
			else
				MessageBox.Show("Save successful.");

			// Disconnect from Origin.
			app = null;
		}
    }
}
